@@ -2751,3 +2751,37 @@ tp_base_client_get_channel_factory (TpBaseClient *self)
 
   return self->priv->channel_factory;
 }
+
+/**
+ * tp_base_client_is_handling_channel:
+ * @self: a #TpBaseClient
+ * @channel: a #TpChannel
+ *
+ * Check if @self is currently handling @channel.
+ *
+ * Returns: %TRUE if @self is handling @channel, %FALSE otherwise
+ * Since: 0.14.5
+ */
+gboolean
+tp_base_client_is_handling_channel (TpBaseClient *self,
+    TpChannel *channel)
+{
+  GList *channels, *l;
+  gboolean found = FALSE;
+
+  g_return_val_if_fail (TP_IS_BASE_CLIENT (self), FALSE);
+  g_return_val_if_fail (self->priv->flags & CLIENT_IS_HANDLER, FALSE);
+
+  channels = tp_base_client_get_handled_channels (self);
+  for (l = channels; l != NULL && !found; l = g_list_next (l))
+    {
+      TpChannel *chan = l->data;
+
+      if (!tp_strdiff (tp_proxy_get_object_path (channel),
+            tp_proxy_get_object_path (chan)))
+        found = TRUE;
+    }
+
+  g_list_free (channels);
+  return found;
+}
